import os
import coloredlogs, logging
import time
import datetime
import argparse as arg
import dask.dataframe as dd
from dataset_descriptor.src.utils import *
import warnings

warnings.filterwarnings("ignore")

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG', logger=logger)
coloredlogs.install(level='success', logger=logger)

# Get execution's parameters 
parser = arg.ArgumentParser()

parser.add_argument("-f","--file",default="__n",type=str,help="Argumento donde se especifica el archivo o carpeta a procesar.\n"+\
                            "Si fuera una carpeta, se procesaran todos los archivos incluidos en ella.")

parser.add_argument('-o','--output',default='',type=str,help="Archivo(Carpeta) de salida la cual tendra el(los) reporte(s) con las descripciones "+\
                        "de los archivos procesados.")
parser.add_argument('--header',action='store_true',help="Indicar si el o los archivos presentan header.")
parser.add_argument('-s','--split',action='store_true',help="Si este flag esta presente, entonces el proceso construira un informe (archivo) para "+
                                    "cada archivo procesado.")

args = vars(parser.parse_args())
if args["file"]=='__n':
    logger.info("\nPor favor seleccione un archivo o carpeta para procesar con el comando -f o --file. Por ejemplo: '--file input1.csv'\n")
    exit(0)
if args["split"]:
    if args["output"]=='':
        logger.info("\nPor favor seleccione un nombre de carpeta donde se alojaran los informes. Esto es requerido al usar el flag -s o --split.\n")
        exit(0)

# Run function

def run():
    start_time = datetime.datetime.now()
    tic = time.time()
    logger.info("Iniciando proceso de Análisis de Archivos. Hora actual: "+start_time.strftime(cfg.datetime_format_string))
    logger.info("El token asignado es :'"+cfg.sep+"'")
    try:
        if os.path.isdir(args["file"]):
            logger.info("Se procesaran los archivos de la carpeta: "+args["file"])
            dfs,files = proccess_dir(args["file"],args["header"])
            logger.info("Se encontraron "+str(len(dfs))+" archivos. Creando Informe ...")
            write_from_dir(dfs,files,args["output"],args["file"],args["split"])
        elif os.path.isfile(args["file"]):
            logger.info("Se procesara el archivo: "+args["file"])
            df = proccess_file(args["file"],args["header"])
            write_from_file(df,args["output"],args["file"])
        else:
            logger.info("No se encuentra el archivo especificado")
            exit(0)
        logger.info("Fin de proceso, tomó "+str(round(time.time() - tic,3))+" segundos")    

    except Exception as error:
        print(type(error))
        logger.error("Existe un problema en el proceso : %s", error)
    
