import datetime
import os
import json
import time
import numpy as np
import pandas as pd
import dask.dataframe as dd
from pandas.io.parsers import count_empty_vals
from dataset_descriptor import config as cfg
from os import path
import enum
import coloredlogs, logging
logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG', logger=logger)

# Clase Enum no deja incluir atributo datetime64[ns] y también no se puede repetir 2 atributos con el mismo valor.
class DataType(enum.Enum):
    str          = "Alfanumerico"
    int          = "Entero"
    float        = "Decimal"
    datetime     = "Fecha"
    Moneda       = "Moneda"
    vacio        = "Vacío"

class Moneda:
    def __init__(self,value,simbolo):
        self.value = value
        self.simbolo = simbolo
    def __str__(self):
        return self.value

try:
    with open(cfg.coins_file,'r',encoding="utf8") as file:
        coins = json.load(file)
except FileNotFoundError as er:
    logger.error("No se encuetra el archivo de símbolos de monedas. Por favor configúrelo en el archivo de configuración")
    exit(0)
    

def set_data_type(x):           
    for tp in [float,int,datetime.datetime.strptime,str]:
        try:
            if tp==float and len(str(x).split("."))==1 : continue
            if tp==float and len(str(x).split("."))==2 and int(str(x).split(".")[-1])==0: continue
            if tp==datetime.datetime.strptime:
                for pars in cfg.parse_dates:
                    try: 
                        x = tp(x,pars)
                        break
                    except Exception as err:
                        continue
            else:
                x = tp(x)
            if(type(x).__name__==DataType.str.name): 
                for i in coins["Monedas"]:
                    key = list(i.keys())[0]
                    if x.startswith(i[key]):
                        _ = float(x.replace(i[key],"").replace(",",""))
                        x = Moneda(x,i[key])
                        break
            break
        except ValueError as er:
            continue
        except TypeError as er:
            continue
    return x

def proccess_file(file_path,header):
    if header: 
        df = dd.read_csv(file_path,sep=cfg.sep)
    else: 
        df = dd.read_csv(file_path,sep=cfg.sep,header=None)
        df.columns = df.columns +1
    news = df.compute()
    # Get type data column 
    for col in news.columns:
        news[col] = news[col].replace(np.nan,"")
        news[col] = news[col].apply(set_data_type)
    news = dd.from_pandas(news,npartitions=3)
    
    return news

def proccess_dir(dir_path,header):
    files = os.listdir(dir_path)
    finals = []
    for file in files:
        df = proccess_file(dir_path+"/"+file,header)
        finals.append(df)
    return finals,files


def write_from_file(df,output_file='',input_file='',output_dir='',split=True):
    data = {}
    time_ = datetime.datetime.now()
    data.update({"Archivo Procesado":input_file,"Hora Procesado":time_.strftime(cfg.datetime_format_string),"Columnas":[]})
    if output_file.find(".")!=-1 and len(output_file)>1:
        logger.info("Creando archivo json")
    else:
        logger.info("No hay archivo de salida seleccionado, mostrando solo en pantalla")
    start = time.time()
    
    for num_col,col in enumerate(df.columns):
        min_lenght,max_lenght = '',''
        min_value,max_value,mean_value = '','',''
        nan_values,no_nan_values = '',''
        counts = ''

        if df[col].dtype=='object':
            data_type = [i.__name__ for i in df[col].apply(type).compute().value_counts().index.values]
            if len(data_type)>1:
                types = [DataType[i].value for i in data_type]
                types = ','.join(types)

                nan_values = df[col].isna().sum().compute()
                no_nan_values = df[col].notnull().count().compute() - nan_values
                counts = [i for i in df[col].apply(type).compute().value_counts().values]
                counts = {DataType[i].value:j for i,j in zip(data_type,counts)}
                data["Columnas"].append({"Columna":col,"Caracteristicas":[{"Tipos de Dato":types,"Tipo":[],
                "Valores Nulos":str(nan_values),"Valores No Nulos":str(no_nan_values)}]})
                for i,j in enumerate(data_type):
                    data["Columnas"][num_col]["Caracteristicas"][0]["Tipo"].append({DataType[j].value:[]})
                    aux = [type(x).__name__==j for x in df[col].compute()]
                    aux = df[col].compute().loc[aux]
                    count = aux.shape[0]
                    random_row = aux.sample(n=1).values[0]
                    if j==DataType.int.name or j==DataType.float.name:
                        if aux.isnull().all():
                            min_value,max_value,mean_value = '','', ''
                            min_lenght = ''
                            max_lenght = ''
                            random_row = ''
                            data["Columnas"][num_col]["Caracteristicas"][0]["Tipo"][i][DataType[j].value].append({"Cantidad":str(count),"Mínimo Valor":min_value,
                            "Máximo Valor":max_value,"Valor Promedio":mean_value,"Mínimo dígitos":min_lenght,
                            "Máximo dígitos":max_lenght,"Valor Ejemplo":random_row})
                        elif j==DataType.float.name:
                            integer_part = aux.apply(lambda x:str(float(x))).apply(lambda x: x.split(".")[0])
                            decimal_part = aux.apply(lambda x:str(float(x))).apply(lambda x: "" if x.split(".")[1]=="0" else x.split(".")[1])
                            min_value,max_value,mean_value = aux.min(), aux.max(), round(aux.mean(),2)
                            min_lenght, max_lenght = integer_part.str.len().min(), integer_part.str.len().max()
                            min_decimal, max_decimal = decimal_part.str.len().min(), decimal_part.str.len().max()
                            data["Columnas"][num_col]["Caracteristicas"][0]["Tipo"][i][DataType[j].value].append({"Cantidad":str(count),"Mínimo Valor":str(min_value),
                            "Máximo Valor":str(max_value),"Valor Promedio":str(mean_value),"Mínimo dígitos enteros":str(min_lenght),
                            "Máximo dígitos enteros":str(max_lenght),"Mínimo dígitos decimales":str(min_decimal),"Máximo dígitos decimales":str(max_decimal),
                            "Valor Ejemplo":str(random_row)})
                        else:
                            integer_part = aux.apply(lambda x:str(float(x))).apply(lambda x: x.split(".")[0])
                            min_value,max_value,mean_value = aux.min(), aux.max(), round(aux.mean(),2)
                            min_lenght, max_lenght = integer_part.str.len().min(), integer_part.str.len().max()
                            data["Columnas"][num_col]["Caracteristicas"][0]["Tipo"][i][DataType[j].value].append({"Cantidad":str(count),"Mínimo Valor":str(min_value),
                            "Máximo Valor":str(max_value),"Valor Promedio":str(mean_value),"Mínimo dígitos":str(min_lenght),
                            "Máximo dígitos":str(max_lenght),"Valor Ejemplo":str(random_row)})

                    elif j==DataType.datetime.name:
                        min_value,max_value = aux.min().strftime(cfg.datetime_format_string), aux.max().strftime(cfg.datetime_format_string)
                        random_row = random_row.strftime(cfg.datetime_format_string)
                        data["Columnas"][num_col]["Caracteristicas"][0]["Tipo"][i][DataType[j].value].append({"Cantidad":str(count),"Fecha Mínima":str(min_value),
                        "Fecha Máxima":str(max_value),"Valor Ejemplo":str(random_row)})
                        
                    elif j==DataType.Moneda.name:                        
                        integer_part =  aux.apply(lambda x:str(x).replace(x.simbolo,"").replace(",","")).astype(float).astype(str).\
                                    apply(lambda x: x.split(".")[0])
                        decimal_part =  aux.apply(lambda x:str(x).replace(x.simbolo,"").replace(",","")).astype(float).astype(str).\
                                    apply(lambda x: "" if x.split(".")[1]=="0" else x.split(".")[1])
                        min_lenght, max_lenght = integer_part.str.len().min(), integer_part.str.len().max()
                        min_decimal, max_decimal = decimal_part.str.len().min(), decimal_part.str.len().max()
                        value_part = aux.apply(lambda x:str(x).replace(x.simbolo,"").replace(",","")).astype(float)
                        min_value = random_row.simbolo+ format(value_part.min(),',.3f')
                        max_value = random_row.simbolo+ format(value_part.max(),',.3f')

                        data["Columnas"][num_col]["Caracteristicas"][0]["Tipo"][i][DataType[j].value].append({"Cantidad":str(count),
                        "Valor Mínimo":min_value,"Valor Máximo":max_value,"Mínimo dígitos enteros":str(min_lenght),"Máximo dígitos enteros":str(max_lenght),
                        "Mínimo dígitos decimales":str(min_decimal),"Máximo dígitos decimales":str(max_decimal),"Valor Ejemplo":str(random_row)})
                    else:
                        min_lenght = aux.astype(str).str.len().min()
                        max_lenght = aux.astype(str).str.len().max()
                        data["Columnas"][num_col]["Caracteristicas"][0]["Tipo"][i][DataType[j].value].append({"Cantidad":str(count),
                        "Mínima Longitud":str(min_lenght),"Máxima Longitud":str(max_lenght),"Valor Ejemplo":str(random_row)})
                
                    
            else:
                counts = df[col].count().compute()
                if df[col].apply(lambda x:True if x=="" else False).compute().all():
                    min_lenght,max_lenght = '',''
                    nan_values, no_nan_values = counts, 0
                    random_row = ""
                    data_type = DataType.vacio.name
                else:
                    data_type = str(data_type[0])
                    min_lenght = df[col].astype(str).str.len().min().compute()
                    max_lenght = df[col].astype(str).str.len().max().compute()
                    nan_values = df[col].isna().sum().compute()
                    no_nan_values = df[col].notnull().count().compute() - nan_values
                    random_row = df[col].sample(n=1).values.compute()[0]
                data["Columnas"].append({"Columna":col,"Características":[{"Tipo de Dato":DataType[data_type].value,"Cantidad de Datos":str(counts),
                "Mínima Longitud":str(min_lenght),"Máxima Longitud":str(max_lenght),"Valores Nulos":str(nan_values),"Valores No Nulos":str(no_nan_values),
                "Valor Ejemplo":str(random_row)}]})

            
        elif df[col].dtype==DataType.datetime.name or df[col].dtype=='datetime64[ns]':
            data_type = DataType.datetime.name
            
            min_value,max_value = df[col].min().compute().strftime(cfg.datetime_format_string), df[col].max().compute().strftime(cfg.datetime_format_string)
            
            counts = df[col].count().compute()
            nan_values = df[col].isna().sum().compute()
            no_nan_values = df[col].notnull().count().compute() - nan_values
            random_row = df[col].sample(n=1).values.compute()
            random_row = pd.to_datetime(str(random_row[0])).strftime(cfg.datetime_format_string)
            data["Columnas"].append({"Columna":col,"Características":[{"Tipo de Dato":DataType[str(data_type)].value,"Cantidad de Datos":str(counts),
            "Mínima Fecha":str(min_value),"Máxima Fecha":str(max_value),"Valores Nulos":str(nan_values),"Valores No Nulos":str(no_nan_values),
            "Valor Ejemplo":str(random_row)}]})
            

        elif df[col].dtype==DataType.Moneda.name:
            data_type = df[col].dtype
            random_row = df[col].sample(n=1).values.compute()
            integer_part =  df[col].apply(lambda x:str(x).replace(x.simbolo,"").replace(",","")).astype(float).astype(str).\
                                    apply(lambda x: x.split(".")[0])
            decimal_part =  df[col].apply(lambda x:str(x).replace(x.simbolo,"").replace(",","")).astype(float).astype(str).\
                                    apply(lambda x: "" if x.split(".")[1]=="0" else x.split(".")[1])
            min_lenght, max_lenght = integer_part.str.len().min(), integer_part.str.len().max()
            min_decimal, max_decimal = decimal_part.str.len().min(), decimal_part.str.len().max()
            value_part = df[col].apply(lambda x:str(x).replace(x.simbolo,"").replace(",","")).astype(float)
            min_value = random_row.simbolo+ format(value_part.min(),',.3f')
            max_value = random_row.simbolo+ format(value_part.max(),',.3f')

            data["Columnas"].append({"Columna":col,"Características":[{"Tipo de Dato":DataType[str(data_type)].value,"Cantidad de Datos":str(counts),
            "Valor Mínimo":min_value,"Valor Máximo":max_value,"Mínimo dígitos enteros":str(min_lenght),"Máximo dígitos enteros":str(max_lenght),
            "Valores Nulos":str(nan_values),"Valores No Nulos":str(no_nan_values),
            "Mínimo dígitos decimales":str(min_decimal),"Máximo dígitos decimales":str(max_decimal),"Valor Ejemplo":str(random_row[0])}]})

        else:
            data_type = DataType.int.name if df[col].dtype=="int64" else df[col].dtype
            data_type = DataType.float.name if df[col].dtype=="float64" else data_type
            random_row = df[col].sample(n=1).values.compute()[0]
            counts = df[col].count().compute()
            nan_values = df[col].isna().sum().compute()
            no_nan_values = df[col].notnull().count().compute() - nan_values
            min_value,max_value,mean_value = df[col].min().compute(), df[col].max().compute(), round(df[col].mean().compute(),2)
            if df[col].isnull().compute().all():
                min_value,max_value,mean_value = '', '', ''
                random_row = ''
                min_lenght = ''
                max_lenght = ''
                data_type = 'vacio'
                data["Columnas"].append({"Columna":col,"Características":[{"Tipo de Dato":DataType[str(data_type)].value,"Cantidad de Datos":str(counts),
                "Mínimo Valor":str(min_value),
                "Máximo Valor":str(max_value),"Valor Promedio":str(mean_value),"Valores Nulos":str(nan_values),"Valores No Nulos":str(no_nan_values),
                "Valor Ejemplo":str(random_row)}]})
            elif df[col].dtype==DataType.float.name:
                
                integer_part = df[col].compute().apply(lambda x:str(float(x))).apply(lambda x: x.split(".")[0])
                decimal_part = df[col].compute().apply(lambda x:str(float(x))).apply(lambda x: "" if x.split(".")[1]=="0" else x.split(".")[1])
                min_lenght, max_lenght = integer_part.str.len().min(), integer_part.str.len().max()
                min_decimal, max_decimal = decimal_part.str.len().min(), decimal_part.str.len().max()
            
                data["Columnas"].append({"Columna":col,"Características":[{"Tipo de Dato":DataType[str(data_type)].value,"Cantidad de Datos":str(counts),
                "Mínimo dígitos enteros":str(min_lenght),"Máximo dígitos enteros":str(max_lenght),"Mínimo dígitos decimales":str(min_decimal),
                "Máximo dígitos decimales":str(max_decimal),"Mínimo Valor":str(min_value),"Máximo Valor":str(max_value),
                "Valor Promedio":str(mean_value),"Valores Nulos":str(nan_values),"Valores No Nulos":str(no_nan_values),
                "Valor Ejemplo":str(random_row)}]})
            else:
                integer_part = df[col].compute().apply(lambda x:str(float(x))).apply(lambda x: x.split(".")[0])
                min_lenght, max_lenght = integer_part.str.len().min(), integer_part.str.len().max()
                data["Columnas"].append({"Columna":col,"Características":[{"Tipo de Dato":DataType[str(data_type)].value,"Cantidad de Datos":str(counts),
                "Mínimo dígitos":str(min_lenght),"Máximo dígitos":str(max_lenght),"Mínimo Valor":str(min_value),
                "Máximo Valor":str(max_value),"Valor Promedio":str(mean_value),"Valores Nulos":str(nan_values),"Valores No Nulos":str(no_nan_values),
                "Valor Ejemplo":str(random_row)}]})
    
    if split:
        
        print(json.dumps(data,indent=3,ensure_ascii=False))   


    if output_file.find(".")!=-1 and len(output_file)>2:
        if output_dir!='':
            with open(output_dir+"/"+output_file,'w') as file:   
                json.dump(data,file,indent=4)
                logger.info("Archivo "+output_file+" procesado y guardado en la carpeta "+output_dir)
        else:
            if split:
                if os.path.isdir(output_file)==False:
                    os.makedirs(os.path.dirname(output_file))
                with open(output_file,'w') as file:   
                    json.dump(data,file,indent=4)
                    logger.info("Archivo procesado y guardado en "+output_file)
            else:
                return data

    logger.info("Fin de creación de archivo json, tomó "+str(round(time.time() - start,3))+" segundos") 
    


        

def write_from_dir(dfs,files,output,input_dir,split_files):
    if split_files and output!='':
        if os.path.exists(output)==False: os.makedirs(output) 
        for df,file in zip(dfs,files):
            if output!='':
                logger.info("Procesando archivo "+file+". Se genera el archivo de salida "+file+cfg.sufijo+" y se guardara en la carpeta "+output)
                write_from_file(df,file+cfg.sufijo,file,output)
            else:
                write_from_file(df,'',file)
    else:

        final = {}
        final.update({"Archivos_Procesados":[]})
        for df,file in zip(dfs,files):
            logger.info("Procesando archivo "+file)
            aux = write_from_file(df,output,file,'',False)
            final["Archivos_Procesados"].append(aux)
        print(json.dumps(final,indent=3,ensure_ascii=False))














